;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; program - Making rotated solid object around
;;           arbitrary axis using Bezier curve
;;           without OpenGL using.
;;
;; author  - Maciej Guba aka macgub
;; email   - macgub3@wp.pl
;; web     - macgub.hekko.pl
;;
;; Program makes rotared solids with Bezier curve based wall.
;; Green lines with red bars on begin and end - curve derives.
;; Line with bright blue bars on begin and end - axis.
;; You can modify curve and axis using mouse (left button on
;; bar and move). Saving to 3ds file 'object.3ds' - press TAB.
;; For now there are problems with 3ds compatibility, only my
;; viewers see it (to download 3ds viewer search my web page).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
format PE GUI 4.0
entry start

include '/include/win32a.inc'

XRES	equ 1280 ;1600 ;1200 ;800      ;choose screen size
YRES	equ 1024 ; 900 ; 675 ;450
SIZE_X	equ XRES
SIZE_Y	equ YRES
ROUND	equ 8
ROTARY_STEPS = 20   ; ROTARY_STEPS and STEPS  must have the same
		    ; value (both determines object quality
STEPS = 20	    ; -> should be divisable by 4 or 8 in AVX cause
NON  = 0
MMX  = 1
SSE  = 2
SSE2 = 3
SSE3 = 4
AVX  = 5	    ; avx version not ready to run
Ext = SSE2


section '.text' code readable executable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; start and message loop
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
start:
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax
    invoke LoadIcon,0,IDI_APPLICATION
    mov [wc.hIcon],eax
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    mov [bmi.biSize],sizeof.BITMAPINFOHEADER
    mov [bmi.biWidth],XRES
    mov [bmi.biHeight],-YRES
    mov [bmi.biPlanes],1
    mov [bmi.biBitCount],32
    mov [bmi.biCompression],BI_RGB

    mov [bmi.biSizeImage],0
    mov [bmi.biXPelsPerMeter],0
    mov [bmi.biYPelsPerMeter],0
    mov [bmi.biClrUsed],0
    mov [bmi.biClrImportant],0

    mov [rect.left],0
    mov [rect.top],0
    mov [rect.right],XRES
    mov [rect.bottom],YRES
    invoke AdjustWindowRect,rect,WS_POPUP or WS_SYSMENU or WS_CAPTION,0 

    invoke RegisterClass,wc

    mov ecx,[rect.bottom]
    mov eax,[rect.right]
    sub ecx,[rect.top]
    sub eax,[rect.left] 

    invoke CreateWindowEx,0,_class,_title,WS_OVERLAPPEDWINDOW\
	   xor WS_THICKFRAME xor WS_MAXIMIZEBOX,0,0,eax,ecx,NULL,NULL,0,0
    mov [hwnd],eax
    invoke GetDC,[hwnd]
    mov [hdc],eax
    invoke ShowWindow,[hwnd],SW_SHOW

    fninit
    fldpi		      ; init needed variables
    fadd   st,st0
    fidiv  [rotary_steps]
    fstp   [one_deg]



    invoke timeGetTime
    mov [s_time],eax	     ;initialize start time

msg_loop:
    invoke timeGetTime
    sub eax,[s_time]
    mov [e_time],eax	     ;get new end time

;    stdcall g_draw
    call   draw_all   ; draw crucial lines and Bezier curve
    invoke StretchDIBits,[hdc],0,0,XRES,YRES,0,0,XRES,YRES,buffer,bmi,0,SRCCOPY
    invoke Sleep,2


    invoke PeekMessage,msg,0,0,0,PM_NOREMOVE
    or eax,eax
    je msg_loop

    invoke GetMessage,msg,NULL,0,0
    or eax,eax
    je end_loop
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg

    jmp msg_loop

error:
    invoke MessageBox,NULL,_error,NULL,MB_ICONERROR+MB_OK

end_loop:
    invoke ReleaseDC,[hwnd],[hdc]
    invoke DestroyWindow,[hwnd]
    invoke ExitProcess,[msg.wParam]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; WindowProc
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
proc WindowProc uses ebx esi edi, hwnd,wmsg,wparam,lparam
     cmp [wmsg],WM_KEYDOWN
     je .wmkeydown
     cmp   [wmsg],WM_LBUTTONDOWN
     jne   @f
     mov   ecx,[lparam]
     mov   eax,ecx

     movzx ebx,ax
     shr   eax,16
     mov   edx,XRES
     mul   edx
     add   eax,ebx
     shl   eax,2
     add   eax,buffer
     mov   ebx,[eax]
     and   ebx,0xff000000
     cmp   ebx,0xff000000
     je    @f
     shr   ebx,24
     mov   [crucial_point],ebx
     mov   [move_flag],1

   @@:
     cmp   [wmsg],WM_MOUSEMOVE
     jne   .next_msg
     cmp   [move_flag],1
     jne   .next_msg
     mov   ecx,[lparam]
     mov   ebx,[crucial_point]
     cmp   ebx,253		   ; check if axis
     jae   @f
     shl   ebx,2
     mov   [ebx+ipx1],ecx
     jmp   .draw
    @@:
     sub   ebx,253
     shl   ebx,2
     mov   [ebx+ix1_axis],ecx
   .draw:
     pushad
     call  draw_all
     popad
  .next_msg:
     cmp   [wmsg],WM_LBUTTONUP
     jne   @f
     mov   [move_flag],0
  @@:

    cmp [wmsg],WM_DESTROY
    je .wmdestroy
  .defwndproc:
    invoke DefWindowProc,[hwnd],[wmsg],[wparam],[lparam]
    jmp .finish
  .wmkeydown:
    cmp [wparam],VK_UP
    jne .vescape
;    stdcall g_init
    xor eax,eax
    jmp .finish
  .vescape:
    cmp [wparam],VK_TAB
    jne @f
    mov [save_flag],1
    pushad
  ;  int3
    call draw_curve  ; init points list
    call save_object
    popad
 @@:
    cmp [wparam],VK_ESCAPE
    jne .finish
  .wmdestroy:
    invoke PostQuitMessage,0
    xor eax,eax
  .finish: 
    ret
endp
draw_all:
     mov   ecx,XRES * YRES
     cld
     mov   eax,0xff000000
     mov   edi,buffer
     rep   stosd

     movzx eax,word[ipy1]
     shl   eax,16
     mov   ax,[ipx1]
     mov   cl,0
     pushad
     call  putpixel
     popad


     movzx eax,[ipy2]
     shl   eax,16
     mov   ax,[ipx2]
     mov   cl,1
     pushad
     call  putpixel
     popad

     movzx eax,[ipy3]
     shl   eax,16
     mov   ax,[ipx3]
     mov   cl,2
     pushad
     call  putpixel
     popad

     movzx eax,[ipy4]
     shl   eax,16
     mov   ax,[ipx4]
     mov   cl,3
     pushad
     call  putpixel
     popad

     fninit
     fild  word[ipx1]
     fstp  [p1x]
     fild  [ipy1]
     fstp  [p1y]
     mov   [p1z],0.0

     fild  [ipx2]
     fstp  [p2x]
     fild  [ipy2]
     fstp  [p2y]
     mov   [p2z],0.0

     fild  [ipx3]
     fstp  [p3x]
     fild  [ipy3]
     fstp  [p3y]
     mov   [p3z],0.0

     fild  [ipx4]
     fstp  [p4x]
     fild  [ipy4]
     fstp  [p4y]
     mov   [p4z],0.0


     fild  word[ix1_axis]
     fstp  dword[x1_axis]
     fild  [iy1_axis]
     fstp  [y1_axis]
     mov   [z1_axis],0.0

     fild  word[ix2_axis]
     fstp  dword[x2_axis]
     fild  [iy2_axis]
     fstp  [y2_axis]
     mov   [z2_axis],0.0

     mov   eax,[ix1_axis]
     mov   cl,253
     pushad
     call  putpixel
     popad

     mov   eax,[ix2_axis]
     mov   cl,254
     pushad
     call  putpixel
     popad

     pushad
     movzx  eax,word[ix1_axis]
     movzx  ebx,[iy1_axis]
     movzx  ecx,word[ix2_axis]
     movzx  edx,[iy2_axis]
     call   linea
     popad



     pushad
     movzx  eax,word[ipx1]
     movzx  ebx,[ipy1]
     movzx  ecx,[ipx2]
     movzx  edx,[ipy2]
     call   linea
     popad

     pushad
     movzx  eax,[ipx3]
     movzx  ebx,[ipy3]
     movzx  ecx,[ipx4]
     movzx  edx,[ipy4]
     call   linea
     popad

     pushad
     call  draw_curve
     popad
ret
putpixel:
  emms
  mov	  ebx,(YRES-5)*65536+XRES-5
  movd	  mm0,ebx
  mov	  ebx,5*65536+5
  movd	  mm1,ebx
  movd	  mm2,eax
  pmaxsw  mm2,mm1
  pminsw  mm2,mm0

  movd	 eax,mm2
  mov	 edx,XRES
  movzx  ebx,ax
  shr	 eax,16
  mul	 edx
  add	 eax,ebx
  shl	 eax,2
  add	 eax,buffer

  mov	  edi,eax	       ; draw line handle
  mov	  al,cl 	       ; cl -> crucial point number
  shl	  eax,24
  cmp	  cl,253
  jae	  .axis
  or	  eax,0x00ff0000
  jmp	  .draw
 .axis:
  or	  eax,0x0000ffff
 .draw:
  sub	  edi,2*XRES*4+20
  mov	  ecx,5
  cld
  rep	  stosd
  add	  edi,XRES*4-20
  mov	  ecx,5
  rep	  stosd
  add	  edi,XRES*4-20
  mov	  ecx,5
  rep	  stosd
  add	  edi,XRES*4-20
  mov	  ecx,5
  rep	  stosd
  add	  edi,XRES*4-20
  mov	  ecx,5
  rep	  stosd

  emms
ret


include 'b_curve.inc'

include 'line.inc'
include 'arbitrary_vector.inc'
include 'matrix.inc'
include '3dmath.inc'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
section '.data' data readable writeable
    _class	 TCHAR 'Bezier',0
    _title	 TCHAR 'Bezier rotary solid  '
    file_name	 db 'object.3ds',0

    if Ext = SSE2
      .exxt    TCHAR	  'SSE2',0
    else
      .exxt    TCHAR	  'AVX',0
    end if
    _error TCHAR 'Startup failed.',0

    up		dd 0
    s_time	dd 0
    e_time	dd 0
;    one_deg  dd   0.024543692
    rotary_steps  dd ROTARY_STEPS
    curve_pointer db 0
    save_flag	  db 0
    screen_width_var dd XRES
include 'b_curve_var_init.inc'
    ipx1:
		dw 10
    ipy1	dw 20
    ipx2	dw 40
    ipy2	dw 400
    ipx3	dw 500
    ipy3	dw 90
    ipx4	dw 600
    ipy4	dw 700

    ix1_axis:
		dw 100
    iy1_axis	 dw 100
;    iz1_axis     dw 0
    ix2_axis:
		dw 400
    iy2_axis	 dw 400
 ;   iz2_axis     dw 0



    msg  MSG
    rect RECT



    wc WNDCLASS 0,WindowProc,0,0,NULL,NULL,NULL,COLOR_BTNFACE+1,NULL,_class
    one_deg	  dd ?
    x1_axis:
		dd ?
    y1_axis	dd ?
    z1_axis	dd ?
    x2_axis:
		dd ?
    y2_axis	dd ?
    z2_axis	dd ?

    byteswritten  dd ?
    file_handle   dd ?
    hwnd	  dd ?
    hdc 	  dd ?
    triangles_count dd ?

    x_axis_v:
		dd ?
    y_axis_v	dd ?
    z_axis_v	dd ?
    matrix	rb 36
    sinus	dd ?
    cosinus	dd ?
    file_size	dd ?



    move_flag	  dd ?	; mark if crucial point is moved
    crucial_point dd ?	; No of cruc. point if it's moved
include  'b_curve_var_uninit.inc'
    counter    dw ?
    bmi      BITMAPINFOHEADER
    buffer:
       rd XRES*YRES
    points:
       rd 65536
    points_r:
       rd 65536
    triangles:
       rd 65536
    file_buffer:
       rd 65536


section '.idata' import data readable writeable
    library kernel32,'KERNEL32',\user32,'USER32',\gdi32,'GDI32',\
    winmm,'WINMM',\comdlg32,'COMDLG32'



    include '\include\api\kernel32.inc'
    include '\include\api\user32.inc'
    include '\include\api\gdi32.inc'
    include '\include\api\comdlg32.inc'

    import winmm,timeGetTime,'timeGetTime'
















